'use strict';

const bhttp = require('bhttp');
const cheerio = require('cheerio');
const moment = require('moment');

function scrapeLatest(html, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });
    const sceneElements = $('.shoot-list .shoot').toArray();

    return sceneElements.map((element) => {
        const sceneLinkElement = $(element).find('.shoot-thumb-title a');
        const href = sceneLinkElement.attr('href');
        const url = `https://kink.com${href}`;
        const shootId = href.split('/')[2];
        const title = sceneLinkElement.text().trim();

        const poster = $(element).find('.adimage').attr('src');
        const photos = $(element).find('.rollover .roll-image').map((photoIndex, photoElement) => $(photoElement).attr('data-imagesrc')).toArray();

        const date = moment.utc($(element).find('.date').text(), 'MMM DD, YYYY').toDate();
        const actors = $(element).find('.shoot-thumb-models a').map((actorIndex, actorElement) => $(actorElement).text()).toArray();
        const stars = $(element).find('.average-rating').attr('data-rating') / 10;

        const timestamp = $(element).find('.video span').text();
        const timestampComponents = timestamp.split(':'); // fix mixed hh:mm:ss and mm:ss format
        const duration = moment.duration(timestampComponents.length > 2 ? timestamp : `0:${timestamp}`).asSeconds();

        return {
            url,
            shootId,
            entryId: shootId,
            title,
            actors,
            date,
            photos,
            poster,
            rating: {
                stars,
            },
            duration,
            site,
        };
    });
}

async function scrapeScene(html, url, shootId, ratingRes, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });

    // const title = $('h1.shoot-title').text().replace(/\ue800/, ''); // fallback, special character is 'like'-heart
    const title = $('h1.shoot-title span.favorite-button').attr('data-title');
    const actorsRaw = $('.shoot-info p.starring');

    const photos = $('.gallery .thumb img').map((photoIndex, photoElement) => $(photoElement).attr('data-image-file')).toArray();
    const trailerVideo = $('.player span[data-type="trailer-src"]').attr('data-url');
    const trailerPoster = $('.player video#kink-player').attr('poster');

    const date = moment.utc($(actorsRaw)
        .prev()
        .text()
        .trim()
        .replace('Date: ', ''),
    'MMMM DD, YYYY')
        .toDate();

    const actors = $(actorsRaw).find('span.names a').map((actorIndex, actorElement) => $(actorElement).text()).toArray();
    const description = $('.shoot-info .description').text().trim();

    const { average: stars } = ratingRes.body;

    const siteName = $('.shoot-logo a').attr('href').split('/')[2];
    const siteSlug = siteName.replace(/\s+/g, '').toLowerCase();

    const tags = $('.tag-list > a[href*="/tag"]').map((tagIndex, tagElement) => $(tagElement).text()).toArray();
    const channel = siteSlug;

    return {
        url,
        shootId,
        entryId: shootId,
        title,
        date,
        actors,
        description,
        photos,
        poster: trailerPoster,
        trailer: {
            src: trailerVideo,
            quality: 480,
        },
        rating: {
            stars,
        },
        tags,
        site,
        channel,
    };
}

async function fetchLatest(site, page = 1) {
    const res = await bhttp.get(`${site.url}/latest/page/${page}`);

    return scrapeLatest(res.body.toString(), site);
}

async function fetchScene(url, site) {
    const shootId = new URL(url).pathname.split('/')[2];

    const [res, ratingRes] = await Promise.all([
        bhttp.get(url),
        bhttp.get(`https://kink.com/api/ratings/${shootId}`),
    ]);

    return scrapeScene(res.body.toString(), url, shootId, ratingRes, site);
}

module.exports = {
    fetchLatest,
    fetchScene,
};
