'use strict';

/* eslint-disable newline-per-chained-call */
const bhttp = require('bhttp');
const cheerio = require('cheerio');
const moment = require('moment');

const fetchSites = require('../sites');
const { matchTags } = require('../tags');

async function getPhotos(entryId, site) {
    const { hostname } = new URL(site.url);

    const res = await bhttp.get(`https://${hostname}/gallery.php?type=highres&id=${entryId}`);
    const html = res.body.toString();

    const $ = cheerio.load(html, { normalizeWhitespace: true });
    const photos = $('a.fakethumb').map((photoIndex, photoElement) => $(photoElement).attr('data-src') || $(photoElement).attr('href')).toArray();

    return photos;
}

async function getChannelSite($, site) {
    const { hostname } = new URL(site.url);

    if (site.isFallback && hostname.match('private.com')) {
        const siteElement = $('.content-wrapper .logos-sites a');
        const siteUrl = siteElement.attr('href').slice(0, -1);
        const siteName = siteElement.text();
        const siteSlug = siteName.replace(/\s+/g, '').toLowerCase();

        const channelSite = await fetchSites({
            slug: siteSlug,
            name: siteName,
            url: siteUrl,
        });

        return channelSite;
    }

    return site;
}

function scrapeLatest(html, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });
    const sceneElements = $('.content-wrapper .scene').toArray();

    return sceneElements.map((element) => {
        const sceneLinkElement = $(element).find('h3 a');

        const url = sceneLinkElement.attr('href');
        const title = sceneLinkElement.text();
        const entryId = url.split('/').slice(-1)[0];

        const date = moment.utc($(element).find('.scene-date'), 'MM/DD/YYYY').toDate();

        const actors = $(element).find('.scene-models a').map((actorIndex, actorElement) => $(actorElement).text()).toArray();
        const likes = Number($(element).find('.scene-votes').text());

        const thumbnailElement = $(element).find('img.img-responsive');
        const photoCount = Number(thumbnailElement.attr('thumbs_num'));
        const poster = thumbnailElement.attr('src');
        const photos = Array.from({ length: photoCount }, (val, index) => thumbnailElement.attr(`src${index + 1}`));

        const scene = {
            url,
            entryId,
            title,
            actors,
            date,
            poster,
            photos,
            rating: {
                likes,
            },
            site,
        };

        return scene;
    });
}

async function scrapeScene(html, url, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });

    const entryId = url.split('/').slice(-1)[0];
    const title = $('.video-wrapper meta[itemprop="name"]').attr('content');

    const date = moment.utc($('.video-wrapper meta[itemprop="uploadDate"]').attr('content'), 'MM/DD/YYYY').toDate();
    const actors = $('.content-wrapper .scene-models-list a').map((actorIndex, actorElement) => $(actorElement).text()).toArray();

    const description = $('.video-wrapper meta[itemprop="description"]').attr('content');
    const [minutes, seconds] = $('.video-wrapper meta[itemprop="duration"]').attr('content').match(/\d+/g);
    const duration = Number(minutes) * 60 + Number(seconds);

    const posterScript = $('script:contains(poster)').html();
    const posterLink = posterScript.slice(posterScript.indexOf('https://'), posterScript.indexOf('.jpg') + 4);
    const poster = $('meta[property="og:image"]').attr('content') || posterLink;

    const trailerElementSrc = $('#videojs-trailer source').attr('src');
    const trailer = $('meta[property="og:video"]').attr('content') || trailerElementSrc;

    const likes = Number($('.content-desc #social-actions #likes').text());

    const rawTags = $('.content-desc .scene-tags a').map((tagIndex, tagElement) => $(tagElement).text()).toArray();

    const [tags, photos, channelSite] = await Promise.all([
        matchTags(rawTags),
        getPhotos(entryId, site),
        getChannelSite($, site),
    ]);

    const scene = {
        url,
        entryId,
        title,
        date,
        actors,
        description,
        duration,
        tags,
        poster,
        photos,
        trailer: {
            src: trailer,
        },
        rating: {
            likes,
        },
        site: channelSite || site,
    };

    return scene;
}

async function fetchLatest(site, page = 1) {
    const { hostname } = new URL(site.url);

    if (hostname.match('private.com')) {
        const res = await bhttp.get(`${site.url}/${page}/`);

        return scrapeLatest(res.body.toString(), site);
    }

    const res = await bhttp.get(`${site.url}/scenes/${page}/`);

    return scrapeLatest(res.body.toString(), site);
}

async function fetchScene(url, site) {
    const res = await bhttp.get(url);

    return scrapeScene(res.body.toString(), url, site);
}

module.exports = {
    fetchLatest,
    fetchScene,
};
