'use strict';

const bhttp = require('bhttp');
const cheerio = require('cheerio');
const moment = require('moment');

const { getPhotos, fetchProfile } = require('./gamma');

function scrape(html, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });
    const scenesElements = $('li[data-itemtype=scene]').toArray();

    return scenesElements.map((element) => {
        const sceneLinkElement = $(element).find('.sceneTitle a');

        const url = `${site.url}${sceneLinkElement.attr('href')}`;
        const title = sceneLinkElement.attr('title');

        const entryId = $(element).attr('data-itemid');

        const date = moment
            .utc($(element).find('.sceneDate').text(), 'MM-DD-YYYY')
            .toDate();

        const actors = $(element).find('.sceneActors a')
            .map((actorIndex, actorElement) => $(actorElement).attr('title'))
            .toArray();

        const [likes, dislikes] = $(element).find('.value')
            .toArray()
            .map(value => Number($(value).text()));

        const poster = $(element).find('.imgLink img').attr('data-original');
        const trailer = `https://videothumb.gammacdn.com/307x224/${entryId}.mp4`;

        return {
            url,
            entryId,
            title,
            actors,
            director: 'Mason',
            date,
            poster,
            trailer: {
                src: trailer,
                quality: 224,
            },
            rating: {
                likes,
                dislikes,
            },
            site,
        };
    });
}

async function scrapeScene(html, url, site) {
    const $ = cheerio.load(html, { normalizeWhitespace: true });
    const json = $('script[type="application/ld+json"]').html();
    const json2 = $('script:contains("dataLayer = ")').html();
    const videoJson = $('script:contains("window.ScenePlayerOptions")').html();

    const data = JSON.parse(json)[0];
    const data2 = JSON.parse(json2.slice(json2.indexOf('[{'), -1))[0];
    const videoData = JSON.parse(videoJson.slice(videoJson.indexOf('{"id":'), videoJson.indexOf('};') + 1));

    const entryId = data2.sceneDetails.sceneId || new URL(url).pathname.split('/').slice(-1)[0];

    const title = data2.sceneDetails.sceneTitle || $('meta[name="twitter:title"]').attr('content');
    const description = data2.sceneDetails.sceneDescription || data.description || $('meta[name="twitter:description"]').attr('content');
    // date in data object is not the release date of the scene, but the date the entry was added
    const date = moment.utc($('.updatedDate').first().text(), 'MM-DD-YYYY').toDate();

    const actors = (data2.sceneDetails.sceneActors || data.actor).map(actor => actor.actorName || actor.name);
    const stars = (data.aggregateRating.ratingValue / data.aggregateRating.bestRating) * 5;

    const duration = moment.duration(data.duration.slice(2).split(':')).asSeconds();

    const siteDomain = $('meta[name="twitter:domain"]').attr('content') || 'allblackx.com'; // only AllBlackX has no twitter domain, no other useful hints available
    const siteSlug = siteDomain && siteDomain.split('.')[0].toLowerCase();
    const siteUrl = siteDomain && `https://www.${siteDomain}`;

    const poster = videoData.picPreview;
    const trailer = `${videoData.playerOptions.host}${videoData.url}`;

    const photos = await getPhotos($('.picturesItem a').attr('href'), siteDomain, site);

    const tags = data.keywords.split(', ');

    return {
        url: `${siteUrl}/en/video/${new URL(url).pathname.split('/').slice(-2).join('/')}`,
        entryId,
        title,
        date,
        actors,
        director: 'Mason',
        description,
        duration,
        poster,
        photos,
        trailer: [
            {
                src: trailer.replace('hd', 'sm'),
                quality: 240,
            },
            {
                src: trailer.replace('hd', 'med'),
                quality: 360,
            },
            {
                src: trailer.replace('hd', 'big'),
                quality: 480,
            },
            {
                // probably 540p
                src: trailer,
                quality: parseInt(videoData.sizeOnLoad, 10),
            },
            {
                src: trailer.replace('hd', '720p'),
                quality: 720,
            },
            {
                src: trailer.replace('hd', '1080p'),
                quality: 1080,
            },
            {
                src: trailer.replace('hd', '4k'),
                quality: 2160,
            },
        ],
        tags,
        rating: {
            stars,
        },
        site,
        channel: siteSlug,
    };
}

async function fetchLatest(site, page = 1) {
    const res = await bhttp.get(`${site.url}/en/videos/AllCategories/0/${page}`);

    return scrape(res.body.toString(), site);
}

async function fetchUpcoming(site) {
    const res = await bhttp.get(`${site.url}/en/videos/AllCategories/0/1/upcoming`);

    return scrape(res.body.toString(), site);
}

async function fetchScene(url, site) {
    const res = await bhttp.get(url);

    return scrapeScene(res.body.toString(), url, site);
}

async function xEmpireFetchProfile(actorName) {
    return fetchProfile(actorName, 'xempire');
}

module.exports = {
    fetchLatest,
    fetchProfile: xEmpireFetchProfile,
    fetchUpcoming,
    fetchScene,
};
